/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "gtkconfigview.h"

#include "../debug.h"
#include "../sofa-i18n.h"
#include "../settings.h"
#include "../menus/menu.h"
#include "../modules/modulemanager.h"
#include "../modules/gtkconfigwindow.h"


static Module * currentModule = NULL;

/***************************
 * GtkConfigView callbacks *
 ***************************/
 
// Called when a module is activated / desactivated
void module_toggled (GtkCellRendererToggle *cell_renderer,
                     gchar                 *pth,
                     gpointer               data){
  log("Toggle %s \n", pth);
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreeIter   iter;
  GtkTreePath  *path = gtk_tree_path_new_from_string(pth);
  
  gtk_tree_model_get_iter (model, &iter, path);
	gtk_list_store_set(GTK_LIST_STORE (model), 
    &iter,
    0, 
    !gtk_cell_renderer_toggle_get_active(cell_renderer),
    -1
    );
  
  //Retrive the pointer to the module
  GValue val;
  val.g_type = G_TYPE_POINTER;
	gtk_tree_model_get_value (model, &iter, 3, &val);
	currentModule = (Module*) g_value_get_pointer(&val);
 
  //Change active in Settings
	std::string key = "/apps/sofa/modules/";
	key.append(currentModule->getSettingName());
	key.append("/active");
	log("key: %s\n", key.c_str());
	SETTINGS->setBool(key.c_str(), !gtk_cell_renderer_toggle_get_active(cell_renderer));
	
}

// Called when a module is selected
static void module_selection_changed(GtkTreeSelection *sel, GtkTreeModel *model) {
	GtkTreeIter  iter;
	GValue val;
	
	if (! gtk_tree_selection_get_selected (sel, &model, &iter))
		return;
  log("Selection changed\n");
	
	val.g_type = G_TYPE_POINTER;
	gtk_tree_model_get_value (model, &iter, 3, &val);
	
	currentModule = (Module*) g_value_get_pointer(&val);
  
  g_value_unset(&val);
}

// Called when the user clicks on Configure
static void config_module(GtkWidget *button, gpointer window)
{
  GtkWidget * gtkWindow = (GtkWidget*) window;
	//Open the modules' ConfigWindow from currentModule
	if(currentModule){
	  GtkConfigWindow *conf = static_cast<GtkConfigWindow*>(currentModule->getConfigWindow());
	  if(conf){
	    conf->show();
	  } else {
	    //Error dialog: this module doesn't support this toolkit
      GtkWidget * dialog = gtk_message_dialog_new (NULL,
                    GTK_DIALOG_DESTROY_WITH_PARENT,
                    GTK_MESSAGE_ERROR,
                    GTK_BUTTONS_CLOSE,
                    _("This module does not support GTK+ configuration windows.")
                    );
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
	  }
	} else {
	  GtkWidget * dialog = gtk_message_dialog_new (NULL,
                    GTK_DIALOG_DESTROY_WITH_PARENT,
                    GTK_MESSAGE_INFO,
                    GTK_BUTTONS_CLOSE,
                    _("Please select a module first.")
                    );
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
	}
}

// Called when the user clicks on About
//static void about_module(GtkWidget *button, gpointer not_used)
//{
	//Open the module's About from currentModule
//	if(currentModule);
//}

// Called when the user selects a new color
static void color_changed(GtkColorButton *button, gpointer key){
  GdkColor color;
  gtk_color_button_get_color(button, &color);
  char * html_color = new char [25];
  sprintf( html_color, "#%02x%02x%02x", color.red>>8, color.green>>8, color.blue>>8 );
  SETTINGS->setString((const char*)key, html_color);
  delete html_color;
}

// Called when the user selects a new font
static void font_changed(GtkFontButton *button, gpointer key){
  std::string font = gtk_font_button_get_font_name(button);
  SETTINGS->setString((const gchar*)key, font.c_str());
}
/************************
 * GtkConfigView Method *
 ************************/
 
GtkConfigView::GtkConfigView (){
 _notebookPage = 0;
}

void GtkConfigView::init ( int argc, char* argv[] ){
  gtk_init (&argc, &argv);
}

void GtkConfigView::show (  ){
  GtkWidget *vbox;
	GtkWidget *bbox;
	GtkWidget *button;
  
  /* Create the window */
	_gtkWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(_gtkWindow), _("Sofa Media Center Configuration"));
	gtk_window_set_resizable (GTK_WINDOW(_gtkWindow), TRUE);
	gtk_container_set_border_width(GTK_CONTAINER(_gtkWindow), 10); // GAIM_HIG_SPACE
	g_signal_connect (G_OBJECT (_gtkWindow), "delete_event",
		    G_CALLBACK (gtk_main_quit), NULL);

  gtk_window_set_default_size( GTK_WINDOW(_gtkWindow), 400, 400);

	vbox = gtk_vbox_new(FALSE, 10); // GAIM_HIG_SPACE
	gtk_container_add(GTK_CONTAINER(_gtkWindow), vbox);
	gtk_widget_show(vbox);

	/* The notebook */
	_notebook = gtk_notebook_new ();
	gtk_box_pack_start (GTK_BOX (vbox), _notebook, FALSE, FALSE, 0);
	gtk_widget_set_size_request (_notebook, 400,400);
	gtk_widget_show(_notebook);
	
  /* The buttons to press! */
	bbox = gtk_hbutton_box_new();
	gtk_box_set_spacing(GTK_BOX(bbox), 10); //GAIM_HIG_BOX_SPACE
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
	gtk_widget_show (bbox);

	button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
	g_signal_connect_swapped(G_OBJECT(button), "clicked",
							 G_CALLBACK(gtk_main_quit), _gtkWindow);
	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	createNotebook();

	/* Show everything. */
	gtk_widget_show(_gtkWindow);
	// start the main loop 
	gtk_main ();


}

void GtkConfigView::createNotebook(){

  notebookAddPage(_("Modules"), createModulePage(),_notebookPage++);
	notebookAddPage(_("Themes"), createThemePage(),_notebookPage++);
	//notebookAddPage(_("Remote & Shortcuts"), createKeyboardPage(),_notebookPage++);
	
}

GtkWidget * GtkConfigView::createModulePage(){
  GtkWidget *ret;
	GtkWidget *sw;
	GtkWidget *view;
	GtkWidget *bbox;
	GtkWidget *button;
	GtkCellRenderer *rend;
	GtkTreeViewColumn *col;
	GtkTreeSelection *sel;
	GtkTreeRowReference *rowref;
	GtkWidget *label;
	GtkTargetEntry te[3] = {{"text/plain", 0, 0},{"text/uri-list", 0, 1},{"STRING", 0, 2}};
	GtkListStore *module_store;

	ret = gtk_vbox_new(FALSE, 10); //GAIM_HIG_CAT_SPACE
	gtk_container_set_border_width (GTK_CONTAINER (ret), 10); //GAIM_HIG_BORDER

	label = gtk_label_new(_("This is the list of modules.  "));

	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);

	gtk_box_pack_start(GTK_BOX(ret), label, FALSE, TRUE, 0);
	gtk_widget_show(label);

	sw = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);

	gtk_box_pack_start(GTK_BOX(ret), sw, TRUE, TRUE, 0);
	module_store = gtk_list_store_new (4, 
	  G_TYPE_BOOLEAN, // Active
	  GDK_TYPE_PIXBUF,// Icon 
	  G_TYPE_STRING,  // Description
	  G_TYPE_POINTER  // Pointer to the Object
	  );

	rowref = moduleList(module_store);

	view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(module_store));

	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
  g_signal_connect (G_OBJECT (sel), "changed",
			  G_CALLBACK (module_selection_changed),
			  module_store);
	
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(module_store),
										 2, GTK_SORT_ASCENDING);
  
  rend = gtk_cell_renderer_toggle_new();
	col = gtk_tree_view_column_new_with_attributes (_("Active"),
							rend,
							"active", 0,
							NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(view), col);
	g_signal_connect(G_OBJECT(rend), "toggled",
							 G_CALLBACK(module_toggled), module_store);

  rend = gtk_cell_renderer_pixbuf_new();
  col = gtk_tree_view_column_new_with_attributes (_("Icon"),
							rend,
							"pixbuf", 1,
							NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(view), col);

	rend = gtk_cell_renderer_text_new();
	col = gtk_tree_view_column_new_with_attributes (_("Description"),
							rend,
							"markup", 2,
							NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW(view), col);
	g_object_unref(G_OBJECT(module_store));
	gtk_container_add(GTK_CONTAINER(sw), view);

	if (rowref) {
		GtkTreePath *path = gtk_tree_row_reference_get_path(rowref);
		gtk_tree_row_reference_free(rowref);
		gtk_tree_selection_select_path(sel, path);
		gtk_tree_path_free(path);
	}
	
  /* The buttons to press! */
	bbox = gtk_hbutton_box_new();
	gtk_box_set_spacing(GTK_BOX(bbox), 10); //GAIM_HIG_BOX_SPACE
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_START);
	gtk_box_pack_start(GTK_BOX(ret), bbox, FALSE, FALSE, 0);
	gtk_widget_show (bbox);
	
	button = gtk_button_new_with_mnemonic (_("_Configure module"));
	g_signal_connect_swapped(G_OBJECT(button), "clicked",
							 G_CALLBACK(config_module), _gtkWindow);
	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
	gtk_widget_show(button);
	
	//button = gtk_button_new_with_mnemonic (_("_About module"));
	//g_signal_connect_swapped(G_OBJECT(button), "clicked",
	//						 G_CALLBACK(about_module), NULL);
	//gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
	//gtk_widget_show(button);
	
	gtk_widget_show_all(ret);

	return ret;
}

GtkWidget * GtkConfigView::createThemePage(){
  GtkWidget *ret, * table, * label, * button;
	GtkSizeGroup *sg;
  GdkColor color;
  std::string font;
  
  ret = gtk_vbox_new(FALSE, 10); //GAIM_HIG_CAT_SPACE
	table = gtk_table_new(3, 2, TRUE); 
	gtk_table_set_homogeneous(GTK_TABLE(table), FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width (GTK_CONTAINER (ret), 10); //GAIM_HIG_BORDER
	gtk_box_pack_start(GTK_BOX(ret), table, FALSE, TRUE, 0);

	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	
  label = gtk_label_new(_("Background color"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/background_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
  g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed), (void*)"/apps/sofa/theme/background_color");
							 
  label = gtk_label_new(_("Menu color"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/menu_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1, 2);
   g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed), (void*) "/apps/sofa/theme/menu_color");
 
  label = gtk_label_new(_("Selected menu color"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/selected_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2, 3);
   g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed),  (void*)"/apps/sofa/theme/selected_color");
							 
  label = gtk_label_new(_("Menu text font"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
	gtk_widget_show(label);
	
	font = SETTINGS->getString("/apps/sofa/theme/menu_font");
  button = gtk_font_button_new_with_font(font.c_str());
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 3, 4);
  g_signal_connect(G_OBJECT(button), "font-set",
							 G_CALLBACK(font_changed),  (void*)"/apps/sofa/theme/menu_font");
 
	gtk_widget_show_all(ret);

	return ret;
}

GtkWidget * GtkConfigView::createKeyboardPage(){
  GtkWidget *ret, * table, * label, * button;
	GtkSizeGroup *sg;
  GdkColor color;
  
  ret = gtk_vbox_new(FALSE, 10); //GAIM_HIG_CAT_SPACE
	table = gtk_table_new(3, 2, TRUE); 
	gtk_table_set_homogeneous(GTK_TABLE(table), FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 10);
	gtk_container_set_border_width (GTK_CONTAINER (ret), 10); //GAIM_HIG_BORDER
	gtk_box_pack_start(GTK_BOX(ret), table, FALSE, TRUE, 0);

	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
	
  label = gtk_label_new(_("Action key"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/background_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
  g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed), (void*)"/apps/sofa/theme/background_color");
							 
  label = gtk_label_new(_("Menu color"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/menu_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1, 2);
   g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed), (void*) "/apps/sofa/theme/menu_color");
 
  label = gtk_label_new(_("Selected menu color"));
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	gtk_widget_show(label);
	
	gdk_color_parse(SETTINGS->getString("/apps/sofa/theme/selected_color"),&color);
  button = gtk_color_button_new_with_color(&color);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2, 3);
   g_signal_connect(G_OBJECT(button), "color-set",
							 G_CALLBACK(color_changed),  (void*)"/apps/sofa/theme/selected_color");
 
	gtk_widget_show_all(ret);

	return ret;
}

void GtkConfigView::notebookAddPage(const char *text,
  		GtkWidget *page,
			int ind) {

#if GTK_CHECK_VERSION(2,4,0)
	gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), page, gtk_label_new(text));
#else
	gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), page, gtk_label_new(text));
	gtk_notebook_page_num(GTK_NOTEBOOK(_notebook), page);
#endif
}

void GtkConfigView::setModuleManager(ModuleManager * manager){
  _modules = manager;
}

GtkTreeRowReference * GtkConfigView::moduleList(GtkListStore * store){
  GdkPixbuf *pixbuf;
	GtkTreeIter iter;
	GtkTreeRowReference *row_ref = NULL;
	bool active;
	std::string key;

	gtk_list_store_clear(store);

	for(int i = 0; i < _modules->count(); i++){
    Module * mod = _modules->getModule(i);
    if (mod){
      char *description = g_strdup_printf(_("<span size='larger' weight='bold'>%s</span>\n"
  						    "<span size='smaller' foreground='dim grey'>Version %s</span>"),
  						    mod->getName(), mod->getVersion() );
  		gtk_list_store_append (store, &iter);
  		
  		key = "/apps/sofa/modules/";
    	key.append(mod->getSettingName());
    	key.append("/active");
    	log("key: %s\n", key.c_str());
    	
      bool active = SETTINGS->getBool(key.c_str());
  		/*
  		 * LEAK - Gentoo memprof thinks pixbuf is leaking here... but it
  		 * looks like it should be ok to me.  Anyone know what's up?  --Mark
  		 */
  		pixbuf = gdk_pixbuf_new_from_file(mod->getMenu()->getImagePath(), NULL);
  		
  	  //Resize it
      if(pixbuf){
        if(gdk_pixbuf_get_width(pixbuf) > 32 || gdk_pixbuf_get_height(pixbuf) > 32){
          pixbuf =  gdk_pixbuf_scale_simple(pixbuf, 32, 32, GDK_INTERP_BILINEAR);
        }
      }
  		gtk_list_store_set(store, &iter,
  				   0, active,     // Active
  				   1, pixbuf,     // Icon
  				   2, description,// Description
  				   3, mod,  // Pointer
  				   -1);
  				   
  		/*if (pixbuf != NULL)
  			g_object_unref(G_OBJECT(pixbuf));
      */
  		g_free(description);
  		
  	}
    
  } 

	return row_ref;
}
