/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef MENU_H
#define MENU_H

#include "../control.h"

#include <sigc++/trackable.h>
#include <sigc++/signal.h>
#include <glib.h>

/**
 * @class Menu
 * @brief The base class for all menus.
 *
 * In the Component design pattern, this is the base to all menu types.
 * This includes menus without sub-menus like the music player.
 */

class Menu: public sigc::trackable {
  // Associations
  // Attributes
  
    
  private:
    gchar * _title;
    gchar * _subtitle;
    gchar * _imagePath;
    float _percentLoaded;
    Menu * _parent;
    bool _selected;
    
  // Operations
  public:
    enum Updates {
      SELECTED_MENU_CHANGED = 0,/** Current selected menu changed */
      HIGHLIGHTED_MENU_CHANGED, /** Current higlighted menu changed */
      LOADING_CHANGED,          /** A menu is loading */
      PROPERTIES_CHANGED,       /** A menu's properties has changed */
      TICK,                     /** Current position in a clip has changed */ 
      CONTROLS_ON,
      CONTROLS_OFF
      
    };
  
    enum Actions {
      UP = 0,
      DOWN, 
      RIGHT,
      LEFT,
      MENU,
      OK,
      ACTION_COUNT /** The number of defined actions */ // should always be last
    };
    
    /**
     * @brief Contructor
     */
    Menu();
    ~Menu();
    
    /**
     * @brief Execute a user action.
     * @param anAction The action that the controler detected.
     */
    virtual void doAction (Actions anAction) = 0;
    
    /**
     * @brief Sets the title.
     * @param title The menu's title.
     */
    void setTitle ( const gchar *  title );
    
    /**
     * @brief Returns the title.
     * @return The menu's title.
     */
    const gchar * getTitle (  );
    
    /**
     * @brief Sets the sub-title.
     *
     * A sub-title is displayed just under the title.
     * @param subtitle The menu's title.
     */
    void setSubtitle ( const gchar * subtitle );
    
    /**
     * @brief Returns the sub-title.
     * @return The menu's sub-title.
     */
    const gchar * getSubtitle (  );
    
    /**
     * @brief Returns how much of this menu is loaded.
     * @return The percent of load completion (between 0 and 1).
     */
    float getPercentLoaded (  );
    
    /**
     * @brief Set how much of this menu is loaded.
     * @param percent The percent of load completion (between 0 and 1).
     */
    void setPercentLoaded ( float percent );
    
    /**
     * @brief Sets this menu as selected.
     */
    void select (  );
    
    /**
     * @brief Unsets this menu as selected.
     */
    void unselect (  );
   
    /**
     * @brief Returns if the menu is selected.
     * @return True if this menu is selected.
     *         False, otherwise.
     */
    bool isSelected (  );
     
    /**
     * @brief Sets the parent menu.
     * @param parent The parent menu.
     */
    void setParent (Menu * parent);
     
    /**
     * @brief Returns the parent menu.
     * @return The parent menu.
     */
    Menu * getParent (  );
     
    /**
     * @brief Returns the path to the menu's icon.
     * @return The path to the menu's icon.
     */
    const gchar * getImagePath(  );

    /**
     * @brief Sets the path to the menu's icon.
     * @param The path to the menu's icon.
     */
    void setImagePath(const gchar * path);
    
  public:
    /** @brief Emited when the menu is updated. 
     * 
     * This signal should be emited when any property of a menu is changed. 
     */ 
    sigc::signal<void, Menu::Updates>  signal_changed;
    
    /** @brief Emited to start the loading of the menu. 
     *
     *  This signal should be emited before we enter this menu.  In is in fact 
     *  what happens when we select a menu.  During the load signal,  
     *  setPercentLoaded() should be called to update the progress on the loading 
     *  time. The current displayed menu will change only when the percentage
     *  hits 1.00 (100%).  If a menu loads instantly, it should simply call
     *  setPercentLoaded(1).
     */ 
    sigc::signal<void>  signal_load;
    
    /** @brief Emited to undload the menu and what ever it created. */ 
    sigc::signal<void>  signal_unload;
};

#endif
