/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "modulemanager.h"
#include "../debug.h"
#include "../settings.h"
#include "../fileutils.h"

#include <assert.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <limits.h> // PATH_MAX
#include <ltdl.h>   // lt_dlOpen and suches
#include <stdlib.h> // entrypoint

typedef void * getModuleFcn (  ); // function called in modules .so

#ifndef MODULE_PATH_ENV
#  define MODULE_PATH_ENV        "MODULE_PATH"
#endif

ModuleManager::ModuleManager(){
  
  /** @todo Add support for more modules locations */
  _path = MODULESDIR ;
}

void ModuleManager::loadModules ( bool activeOnly  ){
  log("Loading modules %s", (activeOnly ? "(active only)": "(all modules)"));
  log("Search path: %s", _path.c_str());
  Module * mod = NULL;
 
  //Search in the _path for modules
  GDir *dir;
  const gchar *file;
  const char  *ext;
	gchar *path;
	
  dir = g_dir_open(_path.c_str(), 0, NULL);
  if (dir){
  	while ((file = g_dir_read_name(dir))){
  		path = g_build_filename(_path.c_str(), file, NULL);
  		
  		if(hasExtension(file, ".la")){
  		  log("Found candidate: %s", file);
  		  std::string key = "/apps/sofa/modules/";
  		  key.append(stripExtension(file));
  		  key.append("/active");
        if(!activeOnly || SETTINGS->getBool(key.c_str())){
          loadModule(file, mod);
        } else
          log("Module %s is not active", file);
      }

  		g_free(path);
  	}

  	g_dir_close(dir);
  }
  
  
}


Module * ModuleManager::getModule ( int pos ){
  std::list<Module*>::const_iterator it;
  int i = 0;
  for(it = _modules.begin(); it != _modules.end(); it++, i++){
    Module * mod = *it;
    if (mod && i == pos){
       return mod;
    }
  } 
}

int ModuleManager::count (  ){
  return _modules.size();
}

bool ModuleManager::loadModule (  std::string filename, Module * mod ){
  log("Loading module %s",filename.c_str());
          
  /** @todo Fix a bug when the getModule method is not found */
  
  // This code is adapted from the book "GNU Autoconf, Automake, and Libtool"
  // Which can be found online at : 
  // http://www.andamooka.org/reader.pl?pgid=autobookautobook_170
  
  char *errormsg = NULL;
  lt_dlhandle module = NULL;
  getModuleFcn *run = NULL;
  int errors = 0;
  

  /* Initialise libltdl. */
  errors = lt_dlinit ();

  /* Set the module search path. */
  if (!errors){
    const char *path = getenv (MODULE_PATH_ENV);

    if (path != NULL)
      errors = lt_dlsetsearchpath (path);
    errors = lt_dlsetsearchpath (_path.c_str());
      
  }
  
  /* Load the module. */
  if (!errors){
    module = lt_dlopenext (filename.c_str());
  }
  
  /* Find the entry point. */
  if (module){
    run = reinterpret_cast<getModuleFcn *>( lt_dlsym (module, "getModule"));

    /* In principle, run might legitimately be NULL, so
        I don't use run == NULL as an error indicator
        in general. */
    //FIXME: errormsg = dlerrordup (errormsg);
    if (errormsg != NULL){
        errors = lt_dlclose (module);
        module = NULL;
        return false;
      }
  } else {
    log ("Coudln't find module %s: %s", filename.c_str(), lt_dlerror() );
    errors = 1;
    return false;
  }

  /* Call the entry point function. */
  if (!errors){
    mod = reinterpret_cast<Module*> ( (*run)() );
    if (mod == NULL){
      errormsg = strdup ("module entry point execution failed");
      return false;
    }else
      log ("Module '%s' loaded", mod->getName());
  }
  
  mod->setHandle(module);
  
  _modules.push_back(mod);
  
  return true;
}

bool ModuleManager::unloadModule (Module * mod ){
  bool ret = false;
  if(mod){
    const gchar * name = mod->getName();
    gchar * localname = new gchar [strlen(name)];
    g_stpcpy(localname, name);
    log("Unloading module %s",localname);
    
    char  *errormsg = NULL;
    int errors = 0;
    lt_dlhandle handle = mod->getHandle();
    errors = lt_dlclose (handle);
    
    //if (errors){
      //errormsg = dlerrordup (errormsg);

      //if (!errormsg){
        //log ("%s: dlerror() failed.", localname);
      //}
    if (!errors){
      log ("Module '%s' unloaded", localname );
      ret = true;
    }else {
      log ("%s: dlerror() failed.", localname);
      ret = false;
    }
  }
  return ret;

}

void ModuleManager::unloadModules(  ){
  log("Unoading modules");
  char *errormsg = NULL;
  int errors = 0;
  
  // Unload each modules 
  std::list<Module*>::const_iterator it;
  for(it = _modules.begin(); it != _modules.end(); ++it){
   unloadModule(*it);
  } 
  
  /* Finished with ltdl now. */
  if (lt_dlexit() != 0){
    //FIXME: errormsg = dlerrordup (errormsg);
  }
  
  if (errormsg){
    log ("%s: %s.", "FIXME:modulename", errormsg);
    free (errormsg);
  }
}
